require 'net/http'

module Crabbs
  class Crawler
    attr_reader :visited, :site_map

    def initialize(options={})
      @visited = []
      @site_map = {}
      @options = options
    end

    def crawl(uri_string)
      recursively_crawl uri_string, @site_map
    end

    private

    def recursively_crawl(uri_string, hash)
      hash[uri_string] = Hash.new

      return if (@visited.include? uri_string)

      links = extract_links uri_string
      @visited << uri_string

      links.each do |link|
        recursively_crawl(link, hash[uri_string])
      end
    end

    def extract_links(uri_string)
      uri = URI.parse(uri_string)
      STDOUT.puts "Visiting: #{uri_string}" if @options[:verbose]
      STDOUT.putc '.' unless @options[:verbose]
      page = Crabbs::Page.new(Net::HTTP.get(uri), uri.to_s)

      page.links
    rescue URI::InvalidURIError
      []
    end
  end
end
